'******************************************************************************************************************************
'* This source code is provided 'as is' and any express or implied warranties, including,                                     *
'* but not limited to, the implied warranties of merchantability and fitness for a particular                                 *
'* purpose are disclaimed.                                                                                                    *
'*                                                                                                                            *
'* In no event shall the author, Crownhill or contributers be liable for any direct, indirect, incidental,                    *
'* special, exemplary, or consequential damages (including, but not limited to,  procurement of substitute                    *
'* goods or services; loss of use, data, or profits;  or business interruption) however caused and on any                     *
'* theory of liability, whether  in contract, strict liability, or tort (including negligence or otherwise)                   *
'* arising in any way out of the use of this source code, even if advised of the possibility of such damage.                  *
'*                                                                                                                            *
'* Any views and conclusions contained in the source code and any documentation are those of the authors                      *
'* and should not be interpreted as representing official policies, either expressed or implied, of Crownhill Associates Ltd. *
'*                                                                                                                            *
'******************************************************************************************************************************
' Hardware PWM routines that make use of the extra CCP peripherals contained on some devices.
' It also allows frequencies upto 65535Hz
' Low end frequencies and high end frequencies depend on the oscillator value used.
' Frequencies may be approximate depending on the oscillator value used and the frequency required.
' This is not due to a limitation of the compiler, but of the peripheral itself.
'
' Note. All PWM peripherals will share a common frequency.
' For use only with version 3.5.5.3 onwards of the Proton Compiler.
'
    Device = 18F25K22                               ' Use a device that has 4 CCP peripherals
    Declare Xtal = 64                               
    Declare Optimiser_Level = 2
    Declare Dead_Code_Remove = 1
'
' Setup which pins are used by the CCP peripherals
' Note that for some of these, config fuse changes will be required
'
    Declare CCP1_Pin = PORTC.2
    Declare CCP2_Pin = PORTC.1   
    Declare CCP3_Pin = PORTB.5
    Declare CCP4_Pin = PORTB.0
    Declare CCP5_Pin = PORTA.4
   
    Include "HPWM.inc"                              ' Load the HPWM macros into the program
    
    Dim Duty As Byte
    Dim Frequency As Word
        
'---------------------------------------------------------------------------
Main: 
    Declare All_Digital = 1    
'
' Adjust the frequency of all the peripherals
'
    For Frequency = 0 To 65535 Step 64      ' Create a loop for frequency adjustment
        HPWM1_SetFreq(Frequency,127)        ' Alter frequency with 50% duty cycle
        HPWM2_SetFreq(Frequency,127)        ' Alter frequency with 50% duty cycle
        HPWM3_SetFreq(Frequency,127)        ' Alter frequency with 50% duty cycle
        HPWM4_SetFreq(Frequency,127)        ' Alter frequency with 50% duty cycle
        HPWM5_SetFreq(Frequency,127)        ' Alter frequency with 50% duty cycle
        DelayMS 1
    Next
'
' Adjust the duty cycles of all peripherals
'    
    HPWM_SetFreq(4000,0)                    ' Set the frequency of all PWM peripherals
    
    HPWM1_Start()                           ' \     
    HPWM2_Start()                           ' | Enable all peripherals
    HPWM3_Start()                           ' |
    HPWM4_Start()                           ' |
    HPWM5_Start()                           ' /
    While 1 = 1                             ' Create an infinite loop        
        For Duty = 0 To 255
            HPWM1_Duty(Duty)                ' \
            HPWM2_Duty(Duty)                ' |
            HPWM3_Duty(Duty)                ' | Adjust the duty cycle of the PWM
            HPWM4_Duty(Duty)                ' |
            HPWM5_Duty(Duty)                ' /
            DelayMS 10
        Next
    Wend
    
Config_Start
 FOSC = HSHP           ' HS oscillator (high power > 16 MHz)
 PLLCFG = On           ' Oscillator multiplied by 4
 PRICLKEN = On         ' Primary clock enabled
 FCMEN = Off           ' Fail-Safe Clock Monitor disabled
 IESO = Off            ' Internal/External Oscillator Switchover mode disabled
 PWRTEN = On           ' Power up timer enabled
 BOREN = SBORDIS       ' Brown-out Reset enabled in hardware only (SBOREN is disabled)
 BORV = 190            ' Brown Out Reset Voltage set to 1.90 V nominal
 WDTEN = Off           ' Watch dog timer is always disabled. SWDTEN has no effect.
 WDTPS = 128           ' Watchdog Timer Postscale 1:128
 CCP2MX = PORTC1       ' CCP2 input/output is multiplexed with RC1
 PBADEN = Off          ' PORTB<5:0> pins are configured as digital I/O on Reset
 CCP3MX = PORTB5       ' P3A/CCP3 input/output is multiplexed with RB5
 HFOFST = On           ' HFINTOSC output and ready status are not delayed by the oscillator stable status
 T3CMX = PORTC0        ' Timer3 Clock Input (T3CKI) is on RC0
 P2BMX = PORTB5        ' ECCP2 B (P2B) is on RB5
 MCLRE = EXTMCLR       ' MCLR pin enabled, RE3 input pin disabled
 STVREN = Off          ' Stack full/underflow will not cause Reset
 LVP = Off             ' Single-Supply ICSP disabled
 XINST = Off           ' Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
 Debug = Off           ' Disabled
 CP0 = Off             ' Block 0 (000800-001FFFh) not code-protected
 CP1 = Off             ' Block 1 (002000-003FFFh) not code-protected
 CP2 = Off             ' Block 2 (004000-005FFFh) not code-protected
 CP3 = Off             ' Block 3 (006000-007FFFh) not code-protected
 CPB = Off             ' Boot block (000000-0007FFh) not code-protected
 CPD = Off             ' Data EEPROM not code-protected
 WRT0 = Off            ' Block 0 (000800-001FFFh) not write-protected
 WRT1 = Off            ' Block 1 (002000-003FFFh) not write-protected
 WRT2 = Off            ' Block 2 (004000-005FFFh) not write-protected
 WRT3 = Off            ' Block 3 (006000-007FFFh) not write-protected
 WRTC = Off            ' Configuration registers (300000-3000FFh) not write-protected
 WRTB = Off            ' Boot Block (000000-0007FFh) not write-protected
 WRTD = Off            ' Data EEPROM not write-protected
 EBTR0 = Off           ' Block 0 (000800-001FFFh) not protected from table reads executed in other blocks
 EBTR1 = Off           ' Block 1 (002000-003FFFh) not protected from table reads executed in other blocks
 EBTR2 = Off           ' Block 2 (004000-005FFFh) not protected from table reads executed in other blocks
 EBTR3 = Off           ' Block 3 (006000-007FFFh) not protected from table reads executed in other blocks
 EBTRB = Off           ' Boot Block (000000-0007FFh) not protected from table reads executed in other blocks
Config_End

