'******************************************************************************************************************************
'* This source code is provided 'as is' and any express or implied warranties, including,                                     *
'* but not limited to, the implied warranties of merchantability and fitness for a particular                                 *
'* purpose are disclaimed.                                                                                                    *
'*                                                                                                                            *
'* In no event shall the author, Crownhill or contributers be liable for any direct, indirect, incidental,                    *
'* special, exemplary, or consequential damages (including, but not limited to,  procurement of substitute                    *
'* goods or services; loss of use, data, or profits;  or business interruption) however caused and on any                     *
'* theory of liability, whether  in contract, strict liability, or tort (including negligence or otherwise)                   *
'* arising in any way out of the use of this source code, even if advised of the possibility of such damage.                  *
'*                                                                                                                            *
'* Any views and conclusions contained in the source code and any documentation are those of the authors                      *
'* and should not be interpreted as representing official policies, either expressed or implied, of Crownhill Associates Ltd. *
'*                                                                                                                            *
'******************************************************************************************************************************
' Hardware PWM routines that make use of the extra CCP peripherals contained on some enhanced 14-bit core devices.
' It also allows frequencies upto 65535Hz
' Low end frequencies and high end frequencies depend on the oscillator value used.
' Frequencies may be approximate depending on the oscillator value used and the frequency required.
' This is not due to a limitation of the compiler, but of the peripheral itself.
'
' Note. All PWM peripherals will share a common frequency.
' For use only with version 3.5.5.3 onwards of the Proton Compiler.
'    
    Device = 16F1939                                ' Choose an enhanced 14-bit core device
    Declare Xtal = 32                               ' Choose the oscillator
    Declare Optimiser_Level = 1                     ' Squeeze the code
    Declare Dead_Code_Remove = 1                    ' Squeeze it som emore
'
' Select the CCP pins for the available peripherals
'    
    Declare CCP1_Pin = PORTC.2
    Declare CCP2_Pin = PORTC.1   
    Declare CCP3_Pin = PORTE.0 
    Declare CCP4_Pin = PORTD.1
   
    Include "HPWM.inc"                              ' Load the HPWM macros into the program
    
    Dim Duty As Byte
    Dim Frequency As Word
        
'---------------------------------------------------------------------------
Main: 
    Declare All_Digital = 1    
'
' Adjust the frequency of all the peripherals
'
    For Frequency = 0 To 65535 Step 64      ' Create a loop for frequency adjustment
        HPWM1_SetFreq(Frequency,127)        ' Alter frequency with 50% duty cycle
        HPWM2_SetFreq(Frequency,127)        ' Alter frequency with 50% duty cycle
        HPWM3_SetFreq(Frequency,127)        ' Alter frequency with 50% duty cycle
        HPWM4_SetFreq(Frequency,127)        ' Alter frequency with 50% duty cycle
        DelayMS 1
    Next
'
' Adjust the duty cycles of all peripherals
'    
    HPWM_SetFreq(4000,0)                    ' Set the frequency of all PWM peripherals
    
    HPWM1_Start()                           ' \     
    HPWM2_Start()                           ' | Enable all peripherals
    HPWM3_Start()                           ' |
    HPWM4_Start()                           ' /
    While 1 = 1                             ' Create an infinite loop        
        For Duty = 0 To 255
            HPWM1_Duty(Duty)                ' \
            HPWM2_Duty(Duty)                ' |
            HPWM3_Duty(Duty)                ' | Adjust the duty cycle of the PWM
            HPWM4_Duty(Duty)                ' /
            DelayMS 10
        Next
    Wend
    


